@@ -371,7 +371,8 @@ class UndertimeArgumentParser(ConfigArgumentParser):
         super().__init__(
             description="pick a meeting time", epilog=__doc__, *args, **kwargs
         )
-        # do not forget to change the manpage and shell completion in
+        # do not forget to change the manpage (undertime.1),
+        # configuration file (undertime.yml) and shell completion in
         # extra/ when you change anything below, consider
         # https://github.com/iterative/shtab
         group = self.add_argument_group("time and date options")
@@ -465,13 +466,6 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             action=NegateAction,
             help=argparse.SUPPRESS,
         )
-        group.add_argument(
-            "-f",
-            "--format",
-            default="fancy_grid_nogap",
-            choices=tabulate.tabulate_formats + ["fancy_grid_nogap"],
-            help="output format (%(default)s)",
-        )
         group.add_argument(
             "--no-table",
             "--table",
@@ -479,6 +473,13 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             action=NegateAction,
             help="hide/show the actual table [default: %(default)s]",
         )
+        group.add_argument(
+            "-f",
+            "--format",
+            default="fancy_grid_nogap",
+            choices=tabulate.tabulate_formats + ["fancy_grid_nogap"],
+            help="output format (%(default)s)",
+        )
         # START HACK
         #
         # ConfigAction doesn't support subparsers and argument groups
@@ -551,6 +552,20 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             self.error("date specified as an argument an option, aborting")
         if ns.datelist and not ns.date:
             ns.date = " ".join(ns.datelist)
+        for h in ("start", "end"):
+            if getattr(ns, h) < 0:
+                # this should probably never happen as it's probably
+                # impossible to pass negative integers through
+                # argparse, but you never know...
+                self.error(
+                    "invalid %s hour (%s), should be a positive integer"
+                    % (h, getattr(ns, h))
+                )
+            if getattr(ns, h) > 24:
+                self.error(
+                    "invalid %s hour (%s), should be less than 24 (midnight)"
+                    % (h, getattr(ns, h))
+                )
         return ns
 
 
@@ -596,7 +611,54 @@ class OffsetZone(pytz._FixedOffset):
         return self._zone
 
 
-def fmt_time_colored(dt, start, end, now):
+def time_in_range(hour: int, start: int, end: int) -> bool:
+    """check that the given hour is in the given range
+
+    This seemingly trivial question is in its own function because it
+    turns out clever people think they should be able to work night
+    shifts and have a range that goes backwards. Since we reuse those
+    checks in a bunch of places, it makes the code much cleaner to
+    have this separate.
+
+    Sorry.
+
+    >>> time_in_range(12, 9, 17)
+    True
+    >>> time_in_range(0, 9, 17)
+    False
+    >>> time_in_range(0, 17, 9)
+    True
+
+    """
+    if start == end:
+        logging.debug(
+            "empty range, always out of range: %s == %s (<> %s)", start, end, hour
+        )
+        return False
+    if start < end:
+        # normal case, say "nine to five", where the start time is
+        # before the end time
+        logging.debug("normal case: start before end: %s <= %s <= %s", start, hour, end)
+        return start <= hour <= end
+    else:
+        # end < start, unusual case, say "five to nine". presume the
+        # caller meant to wrap around midnight. see
+        # https://gitlab.com/anarcat/undertime/-/issues/29
+        logging.debug(
+            "unusual case: end before start: %s <= %s < 24 || 0 <= %s < %s",
+            start,
+            hour,
+            hour,
+            end,
+        )
+        # here we reverse the logic. instead of checking we're inside
+        # the range, we check that we are *outside* the range, namely
+        # that we are between the start time and midnight ("24"), and
+        # between midnight (now "0") and the end time.
+        return (start <= hour < 24) or (0 <= hour <= end)
+
+
+def fmt_time_colored(dt: datetime.datetime, in_range: bool, now: bool) -> str:
     """format given datetime in color
 
     This uses the termcolor module to color it "yellow" if it's
@@ -606,13 +668,13 @@ def fmt_time_colored(dt, start, end, now):
     attrs = []
     if now:
         attrs.append("bold")
-    if start <= dt.hour <= end:
+    if in_range:
         return termcolor.colored(string, "yellow", attrs=attrs)
     else:
         return termcolor.colored(string, attrs=attrs)
 
 
-def fmt_time_ascii(dt, start, end, now):
+def fmt_time_ascii(dt: datetime.datetime, in_range: int, now: int) -> str:
     """format given datetime using plain ascii (no colors)
 
     This will add a star ("*") if "now" is true and an underscode
@@ -621,7 +683,7 @@ def fmt_time_ascii(dt, start, end, now):
     string = "{0:%H:%M}".format(dt.timetz())
     if now:
         return string + "*"
-    if start <= dt.hour <= end:
+    if in_range:
         return string + "_"
     return string
 
@@ -975,8 +1037,10 @@ def compute_table(
         n = 0
         line = []
         for t in [current_time.astimezone(tz=zone) for zone in timezones]:
-            line.append(fmt_time(t, start, end, current_time == now_utc))
-            n += 1 if start <= t.hour <= end else 0
+            line.append(
+                fmt_time(t, time_in_range(t.hour, start, end), current_time == now_utc)
+            )
+            n += 1 if time_in_range(t.hour, start, end) else 0
         if overlap_show:
             line.append(str(n))
         if n >= overlap_min:
@@ -986,8 +1050,8 @@ def compute_table(
             line = []
             n = 0
             for t in [now_utc.astimezone(tz=zone) for zone in timezones]:
-                line.append(fmt_time(t, start, end, True))
-                n += 1 if start <= t.hour <= end else 0
+                line.append(fmt_time(t, time_in_range(t.hour, start, end), True))
+                n += 1 if time_in_range(t.hour, start, end) else 0
             if overlap_show:
                 line.append(str(n))
             if n >= overlap_min:
@@ -1080,9 +1144,7 @@ if __name__ == "__main__":  # pragma: nocover
     except Exception as e:
         logging.error("unexpected error: %s", e)
         if args.debug:
-            logging.warning(
-                'starting debugger, type "c" and enter to continue'
-            )  # noqa: E501
+            logging.warning('starting debugger, type "c" and enter to continue')
             import pdb
             import traceback
 
